﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;

namespace RxJS
{
    public class CustomerOrdersController : ApiController
    {
        private CustomerFactory cf;
        public CustomerOrdersController()
        {
            cf = new CustomerFactory();
        }

        [HttpGet()]
        public Customer CustomerById(string Id)
        {
                    return cf.CustomerById(Id);
        }

        [HttpGet()]
        public IQueryable<SalesOrder> OrdersByCustomerId(string Id)
        {
            return cf.OrdersByCustomerId(Id);
        }

        [HttpGet()]
        public IQueryable<Customer> AllCustomers()
        {
            return cf.AllCustomers();
        }

        [HttpGet()]
        public SalesOrder OrderById(int Id)
        {
            return cf.OrderById(Id);
        }

        [HttpGet()]
        public IQueryable<SalesOrder> AllSalesOrders()
        {
            return cf.AllOrders();
        }

        [HttpGet()]
        public Product ProductById(int Id)
        {
            return cf.ProductById(Id);
        }

        [HttpGet()]
        public IQueryable<Product> AllProducts()
        {
            return cf.AllProducts();
        }                
    }

    public class CustomerFactory
    {
        public Customer CustomerById(string Id)
        {
            Customer cust = null;
            switch (Id)
            {
                case "A123":
                    cust = new RxJS.Customer
                    {
                        Id = Id,
                        FirstName = "Peter",
                        LastName = "Vogel"
                    };
                    break;
                case "B456":
                    cust = new RxJS.Customer
                    {
                        Id = Id,
                        FirstName = "Jan",
                        LastName = "Vogel"
                    };
                    break;
                case "C789":
                    cust = new RxJS.Customer
                    {
                        Id = Id,
                        FirstName = "Jason",
                        LastName = "van de Velde"
                    };
                    break;
            }
            return cust;
        }

        public Product ProductById(int Id)
        {
            Product prod = null;
            switch (Id)
            {
                case 100:
                    prod = new RxJS.Product
                    {
                        Id = Id,
                        Description = "Widget"
                    };
                    break;
                case 200:
                    prod = new RxJS.Product
                    {
                        Id = Id,
                        Description = "Sprocket"
                    };
                    break;
                case 300:
                    prod = new RxJS.Product
                    {
                        Id = Id,
                        Description = "Flange"
                    };
                    break;
            }
            return prod;
        }

        public SalesOrder OrderById(int Id)
        {
            SalesOrder so = null;
            switch (Id)
            {
                case 1001:
                    so = new RxJS.SalesOrder
                    {
                        Id = Id,
                        CustomerId = "A123",
                        OrderDate = DateTime.Now.AddDays(-5),
                        ProductId = 100
                    };
                    break;
                case 1002:
                    so = new RxJS.SalesOrder
                    {
                        Id = Id,
                        CustomerId = "B456",
                        OrderDate = DateTime.Now.AddDays(-10),
                        ProductId = 100
                     };
                    break;
                case 1003:
                    so = new RxJS.SalesOrder
                    {
                        Id = Id,
                        CustomerId = "A123",
                        OrderDate = DateTime.Now.AddDays(-1),
                        ProductId = 200
                    };
                    break;
                case 1004:
                    so = new RxJS.SalesOrder
                    {
                        Id = Id,
                        CustomerId = "B456",
                        OrderDate = DateTime.Now.AddDays(-1),
                        ProductId = 300
                    };
                    break;
                case 1005:
                    so = new RxJS.SalesOrder
                    {
                        Id = Id,
                        CustomerId = "A123",
                        OrderDate = DateTime.Now.AddDays(-50),
                        ProductId = 300
                    };
                    break;
            }
            return so;
        }

        public IQueryable<SalesOrder> AllOrders()
        {
            List<SalesOrder> sos = new List<SalesOrder>();
            sos.Add(OrderById(1001));
            sos.Add(OrderById(1002));
            sos.Add(OrderById(1003));
            sos.Add(OrderById(1004));
            sos.Add(OrderById(1005));
            return sos.AsQueryable();
        }

        public IQueryable<SalesOrder> OrdersByCustomerId(string custId)
        {
            return AllOrders().Where(so => so.CustomerId == custId);
        }

        public IQueryable<SalesOrder> SalesOrdersByProductId(int Id)
        {
            IEnumerable<SalesOrder> sos = AllOrders();
            return sos.Where(so => so.ProductId == Id).AsQueryable();
        }

        public IQueryable<Customer> AllCustomers()
        {
            List<Customer> custs = new List<Customer>();
            custs.Add(CustomerById("A123"));
            custs.Add(CustomerById("B456"));
            custs.Add(CustomerById("C789"));
            return custs.AsQueryable();
        }
        public IQueryable<Product> AllProducts()
        {
            List<Product> prods = new List<Product>();
            prods.Add(ProductById(100));
            prods.Add(ProductById(200));
            prods.Add(ProductById(300));
            return prods.AsQueryable();
        }
    }

    public class Customer
    {
        public string Id { get; set; }
        public string FirstName { get; set; }
        public string LastName { get; set; }
    }

    public class SalesOrder
    {
        public int Id { get; set; }
        public string CustomerId { get; set; }
        public DateTime OrderDate { get; set; }
        public int ProductId { get; set; }
    }

    public class Product
    {
        public int Id { get; set; }
        public string Description { get; set; }
    }
}